// test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include <stdio.h>
#include "ImageWrapper.hpp"

// Global image pointer
IplImage* src = NULL;
IplImage* disp = NULL;

void calcMeanVar(int xMin, int yMin,
				 int xMax, int yMax,
				 float & meanValue, float & varValue)
{
	// Calculate mean and variance
	int nMeasurements = 0;		// number of measurements
	double sumIntensity = 0;	// Sum of intensity values
	double sumIntensitySq = 0;  // Sum of squared intensity values

	// Loop through box area
	for (int y = yMin ; y <= yMax ; ++y)
	{
		for (int x = xMin ; x <= xMax ; ++x)
		{
			// Get colour value
			CvScalar colourValue = cvGet2D(src,y,x) ;
			// Calculate intensity value
			float intensityValue = 
				(colourValue.val[0] + colourValue.val[1] + colourValue.val[2]) / 3. ;
			
			// Sum of intensity values
			sumIntensity += intensityValue;	
			// Sum of squared intensity values
			sumIntensitySq += intensityValue*intensityValue;
			// Number of measurements
			++nMeasurements;
		}
	}

	// Calculate mean
	meanValue = sumIntensity / nMeasurements;
	// Calculate variance
	varValue  = 
		sumIntensitySq / nMeasurements - meanValue * meanValue;
}

void mouseHandler(int event, int x, int y, int flags, void* param)
{
	switch(event)
	{
		case CV_EVENT_MOUSEMOVE:
			
			disp = cvCloneImage(src);	// copy image for display
			
			// set bounding box parameters
			int yMin, yMax;
			int xMin, xMax;

			// Check bounds of image
			// Set bounding box
			if (y < 5)
				yMin = 0;
			else
				yMin = y - 5;
			if (x < 5)
				xMin = 0;
			else
				xMin = x - 5;
			if (y + 6 > src->height)
				yMax = src->height - 1;
			else
				yMax = y + 5;
			if (x + 6 > src->width)
				xMax = src->width - 1;
			else
				xMax = x + 5;

			// Get colour values
			CvScalar colourValue = cvGet2D(src,y,x) ;
			// Calculate intensity
			float intensityValue = 
				(colourValue.val[0] + colourValue.val[1] + colourValue.val[2]) / 3. ;

			// Calculate mean and variance
		    float meanValue, varValue;
			calcMeanVar(xMin, yMin,
						xMax, yMax, 
						meanValue, varValue );

			// Print to command screen
			printf( "(%i,%i) B:%i, G:%i, R:%i, Intensity: %.1f\n",
					x, y,
				    (int)colourValue.val[0],
					(int)colourValue.val[1],
					(int)colourValue.val[2],
					intensityValue
				  );
			printf("(%i,%i),(%i,%i) Mean:%f, Var:%f\n",
				    xMin, yMin, xMax, yMax,	meanValue, varValue );

			// Draw box on image.
			int invIntensity = 255 - (int)meanValue;

			// Draw rectangle on image
			cvRectangle( disp, cvPoint(xMin,yMin), cvPoint(xMax,yMax),
						 cvScalar(invIntensity,invIntensity,invIntensity),  1 );
			
			// Display image
			cvShowImage("src", disp);

			break;
	}
}

IplImage * createHistImg(IplImage* inputImg, CvScalar histColour)
{
	// Input is filtered colour channel / grey value image
	// Output is a histogram image
	// histColour is the histogram colour wanted

	IplImage* outputImg;

	int histSize = 256;		// the number of bins
	
	CvHistogram* hist = cvCreateHist( 1,             // 1 dimention
		                              &histSize,     // number of bins
									  CV_HIST_ARRAY  // standard array
									);
	
	cvCalcHist( &inputImg, hist );	// Calculate histogram from image

	// Get Min and Max Histogram values
	float minH = 0, maxH = 0;
	cvGetMinMaxHistValue( hist, &minH, &maxH);

	// Create image for displaying histogram
	outputImg = cvCreateImage( cvSize(histSize, maxH), // defining the size
		                               IPL_DEPTH_8U, 
									   3 );
	
	// Loop through histogram bins
	for( int i = 0; i < histSize; ++i )
    {
		// Get value of bin
        float bin_val = cvQueryHistValue_1D(hist, i);
        
		// Draw rectangle on image
		cvRectangle( outputImg,					// image to draw on
			         cvPoint(i,maxH - bin_val),	// start point of rectangle
					 cvPoint(i,maxH),			// end point of rectangle
					 histColour,	            // color to display
					 CV_FILLED					// Fill rectangle (can use int for line thickness)
				   );
    }

	return outputImg;

	// clean up histograms
	cvReleaseHist(&hist);
}

int _tmain(int argc, _TCHAR* argv[])
{
	// Load original image
	src = cvLoadImage("wall-smiley.bmp",1);

	// Create Blue channel image
	IplImage* blueImg = cvCreateImage( cvGetSize(src),  // copy size of source image
		                               IPL_DEPTH_8U,    // image depth, i.e. 8 bit
									   1				// Number of channels
									 );

	IplImage* greenImg = cvCreateImage( cvGetSize(src), IPL_DEPTH_8U, 1);
	IplImage* redImg   = cvCreateImage( cvGetSize(src), IPL_DEPTH_8U, 1);

	// Splits images... remember OpenCV is in BGR format
	cvSplit(src,blueImg,greenImg,redImg,NULL);

	// Create Histograms	
	IplImage* bHistImg = NULL;
	IplImage* gHistImg = NULL;
	IplImage* rHistImg = NULL;
	bHistImg = createHistImg(blueImg, cvScalar(255,0,0) );
	gHistImg = createHistImg(greenImg, cvScalar(0,255,0) );
	rHistImg = createHistImg(redImg, cvScalar(0,0,255) );

	// Set up Windows
	cvNamedWindow("src", 0);
	cvMoveWindow("src", 0, 0);
	cvResizeWindow( "src", src->width, src->height);

	cvNamedWindow( "Blue Histogram", 0 );
	cvMoveWindow("Blue Histogram", src->width + 20, 0);
	cvResizeWindow( "Blue Histogram", 256, 200);

	cvNamedWindow( "Green Histogram", 0 );
	cvMoveWindow("Green Histogram", src->width + 20, 220);
	cvResizeWindow( "Green Histogram", 256, 200);

	cvNamedWindow( "Red Histogram", 0 );
	cvMoveWindow("Red Histogram", src->width + 20, 440);
	cvResizeWindow( "Red Histogram", 256, 200);

	// Show images
    cvShowImage( "Blue Histogram", bHistImg );
	cvShowImage( "Green Histogram", gHistImg );
	cvShowImage( "Red Histogram", rHistImg );
	cvShowImage( "src", src );

	// Initialise Mouse Handler for "src" screen
	cvSetMouseCallback( "src", mouseHandler, 0 );

	// Pause so you can see your output
	cvWaitKey(0);
	
	// Clean up memory

	// clean up windows
	cvDestroyAllWindows();		

	// clean up images
	cvReleaseImage( &src );		
	cvReleaseImage( &blueImg );
	cvReleaseImage( &greenImg );
	cvReleaseImage( &redImg );
	cvReleaseImage( &bHistImg );
	cvReleaseImage( &gHistImg );
	cvReleaseImage( &rHistImg );

	return 0;
}